#!/usr/bin/perl
###############################################################################
#    Webmin Sysstats - cpu-lib.pl
#
#    Webmin Sysstats Module 
#    Copyright (C) 2001 by David Bouius
#    Bug reports to: dbouius@users.sourceforge.net
#    $Id: cpu-lib.pl,v 1.9 2001/12/07 09:25:28 gerbier Exp $
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
###############################################################################

sub install_module {
  local ($minfo, $defaults) = ($_[0], $_[1]);

  local (%my_config);

  my $num_of_cpus = &find_num_cpus();

  if (!-r $minfo->{'dbtables'} . "cpu1.rrd") {
    local ($cmd);

    my $cpu_num = 1;
    while ($cpu_num <= $num_of_cpus) {
      $cmd = $config{'rrdtool'};
      $cmd .= " create " . $minfo->{'dbtables'} . "cpu" . $cpu_num . ".rrd";
      $cmd .= " --start 0 --step 60";
      $cmd .= " DS:user:GAUGE:100:0:U";
      $cmd .= " DS:system:GAUGE:100:0:U";
      $cmd .= " DS:nice:GAUGE:100:0:U";
      $cmd .= " DS:idle:GAUGE:100:0:U";
      $cmd .= " RRA:AVERAGE:0.5:1:70";
      $cmd .= " RRA:AVERAGE:0.5:2:750";
      $cmd .= " RRA:AVERAGE:0.5:15:650";
      $cmd .= " RRA:AVERAGE:0.5:50:650";

      `$cmd`;
# todo: error checking
      $cpu_num++;
    }
  }

# make an assumtion here, there will always be at least one cpu
  if (-r $minfo->{'dbtables'} . "cpu1.rrd") {
    $my_config{'sample_rate'} = 1;
    $my_config{'graph_type'} = 1;

    my $cpu_num = 1;
    while ($cpu_num le $num_of_cpus) {
      $my_config{'cpu' . $cpu_num} = join ( ',',
				  $defaults->{'user_colour'},
				  $defaults->{'system_colour'},
				  $defaults->{'idle_colour'},
				  $defaults->{'nice_colour'}
				);
      $cpu_num++;
    }

    &write_config($minfo, \%my_config, $defaults);

    &install_script($minfo, 'cpu.sh');

    return 1;
  }
  return 0;
}

sub remove_module {
}

sub mod_check_runstop {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  return $my_config->{'runstop'};
}

sub mod_runstop {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  $my_config->{'runstop'} = $my_config->{'runstop'} ? '1' : '0';
  &write_config($minfo, $my_config, $defaults);
}

sub mod_check_upgrade {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  if (-r $minfo->{'dbtables'} . "../cpu/cpu.rrd") {
    return 1;
  }
  return 0;
}

sub mod_upgrade {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  local (%old_config);

  if (-r $minfo->{'dbtables'} . "../cpu/cpu.rrd") {
    my $cmd = "cp -f " . $minfo->{'dbtables'} . "../cpu/cpu.rrd " . $minfo->{'dbtables'} . "cpu1.rrd";

    system($cmd);

    if (-r $minfo->{'config_dir'} . "../cpu/module.cfg" ) {
      if (!&read_file($minfo->{'config_dir'} . "../cpu/module.cfg", \%old_config)) {
	&error(&text('ecannot_readconfig', $minfo->{'config_dir'} . "../cpu/module.cfg"));
      }

      $my_config->{'cpu1'} = $old_config{'cpu'};

      &write_config($minfo, $my_config, $defaults);
    }
  }
}

sub graph_data {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  $rrdtool = $config{'rrdtool'};
  $sizex = $config{'graph_sizex'};
  $sizey = $config{'graph_sizey'};

  $rrd_detals  = " -u 100 --rigid -v '% utilization'";

  my $cpu_num = 1;
  while ($my_config->{'cpu' . $cpu_num} ne "") {
    local ($user_colour, $syst_colour, $idle_colour, $nice_colour)
      = split(',', $my_config->{'cpu' . $cpu_num});

    $def_cpu = "cpu" . $cpu_num;
    $def_user = "user" . $cpu_num;
    $def_syst = "syst" . $cpu_num;
    $def_nice = "nice" . $cpu_num;
    $def_idle = "idle" . $cpu_num;

    $rrd_detals .= " DEF:" . $def_user . "=" . $minfo->{'dbtables'} . $def_cpu . ".rrd:user:AVERAGE";
    $rrd_detals .= " DEF:" . $def_syst . "=" . $minfo->{'dbtables'} . $def_cpu . ".rrd:system:AVERAGE";
    $rrd_detals .= " DEF:" . $def_nice . "=" . $minfo->{'dbtables'} . $def_cpu . ".rrd:nice:AVERAGE";
    $rrd_detals .= " DEF:" . $def_idle . "=" . $minfo->{'dbtables'} . $def_cpu . ".rrd:idle:AVERAGE";
    $rrd_detals .= " CDEF:t" . $cpu_num . "=$def_user,$def_syst,$def_idle,$def_nice,+,+,+";
    $rrd_detals .= " CDEF:u" . $cpu_num . "=100,$def_user,t$cpu_num,/,*";
    $rrd_detals .= " CDEF:s" . $cpu_num . "=100,$def_syst,t$cpu_num,/,*";
    $rrd_detals .= " CDEF:n" . $cpu_num . "=100,$def_nice,t$cpu_num,/,*";
    $rrd_detals .= " CDEF:i" . $cpu_num . "=100,$def_idle,t$cpu_num,/,*";

    if ($my_config->{'graph_type'} eq 0) {
      $rrd_detals .= " LINE1:u" . $cpu_num . "#" . $user_colour . ":' $def_cpu User'";
      $rrd_detals .= " LINE1:s" . $cpu_num . "#" . $syst_colour . ":' $def_cpu System'";
      $rrd_detals .= " LINE1:n" . $cpu_num . "#" . $nice_colour . ":' $def_cpu Nice'";
      $rrd_detals .= " LINE1:i" . $cpu_num . "#" . $idle_colour . ":' $def_cpu Idle\\n'";
    }
    $cpu_num++;
  }

  if ($my_config->{'graph_type'} eq 1) {
    local ($user_colour, $syst_colour, $idle_colour, $nice_colour)
      = split(',', $my_config->{'cpu1'});

    local ($utotal, $stotal, $itotal, $ntotal);

    $utotal = "u1";
    $stotal = "s1";
    $ntotal = "n1";
    $itotal = "i1";

    my $cpu_num = 2;
    while ($my_config->{'cpu' . $cpu_num} ne "") {
      $utotal .= ",u" . $cpu_num . ",+";
      $stotal .= ",s" . $cpu_num . ",+";
      $ntotal .= ",n" . $cpu_num . ",+";
      $itotal .= ",i" . $cpu_num . ",+";

      $cpu_num++;
    }

    $cpu_num--;

    if ($cpu_num lt 1) {
      $utotal = "," . $cpu_num . ",/";
      $stotal = "," . $cpu_num . ",/";
      $ntotal = "," . $cpu_num . ",/";
      $itotal = "," . $cpu_num . ",/";
    }

    $rrd_detals .= " CDEF:tu=" . $utotal;
    $rrd_detals .= " CDEF:ts=" . $stotal;
    $rrd_detals .= " CDEF:tn=" . $ntotal;
    $rrd_detals .= " CDEF:ti=" . $itotal;

    $rrd_detals .= " COMMENT:'Type         Max     Min Average\\n'";
    $rrd_detals .= " AREA:tu#" . $user_colour . ":'User  '";
    $rrd_detals .= " GPRINT:tu:MAX:'%6.2lf'";
    $rrd_detals .= " GPRINT:tu:MIN:'%6.2lf'";
    $rrd_detals .= " GPRINT:tu:AVERAGE:'%6.2lf\\n'";
    $rrd_detals .= " STACK:ts#" . $syst_colour . ":'System'";
    $rrd_detals .= " GPRINT:ts:MAX:'%6.2lf'";
    $rrd_detals .= " GPRINT:ts:MIN:'%6.2lf'";
    $rrd_detals .= " GPRINT:ts:AVERAGE:'%6.2lf\\n'";
    $rrd_detals .= " STACK:tn#" . $nice_colour . ":'Nice  '";
    $rrd_detals .= " GPRINT:tn:MAX:'%6.2lf'";
    $rrd_detals .= " GPRINT:tn:MIN:'%6.2lf'";
    $rrd_detals .= " GPRINT:tn:AVERAGE:'%6.2lf\\n'";
    $rrd_detals .= " STACK:ti#" . $idle_colour . ":'Idle  '";
    $rrd_detals .= " GPRINT:ti:MAX:'%6.2lf'";
    $rrd_detals .= " GPRINT:ti:MIN:'%6.2lf'";
    $rrd_detals .= " GPRINT:ti:AVERAGE:'%6.2lf\\n'";
  }

  foreach $graph ('hour', 'day', 'week', 'month') {
    if ($graph eq 'hour') {
      $time = "-3600";
      $title = "'By the hour'";
    } elsif ($graph eq 'day') {
      $time = "-86400";
      $title = "'Daily'";
    } elsif ($graph eq 'week') {
      $time = "-604800";
      $title = "'Weekly'";
    } elsif ($graph eq 'month') {
      $time = "-18748800";
      $title = "'Monthly'";
    }

    $common  = $rrdtool . " graph " . $minfo->{'tmpgname'} . "_" . $graph . ".png";
    $common .= " --start " . $time . " --end -60";
    $common .= " -h " . $sizey . " -w " . $sizex;
    $common .= " -a PNG --title=" . $title;

    $sig = " COMMENT:'\\n' COMMENT:\"Created: `date`\"";
# &error($common . $rrd_detals . $sig);
    system ($common . $rrd_detals . $sig);
  }
}

sub mod_config {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  my $num_of_cpus = &find_num_cpus();

  print "<table width='100%'><tr><td><b>";
  print  $text{'module_subtitle'};
  print "</b></td></tr></table>\n";

  print "<form action='save_config.cgi'>\n";
  print  "<input type='hidden' name='idx' value='$in{'idx'}'>\n";

  print      "<table width='100%'>\n";
  print       "<tr><td><table width='100%'>\n";
  print        "<tr><td>";
  print         "<b>$test{'display_type'}</b>";
  printf "<input type='radio' name='graph_type' value='1' %s> $text{'module_word_bar'}\n",
    $my_config->{'graph_type'} ? "checked" : "";
  printf "<input type='radio' name='graph_type' value='0' %s> $text{'module_word_line'}</td> </tr>\n",
    $my_config->{'graph_type'} ? "" : "checked";
  print        "</td></tr>\n";
  print       "</td></tr></table>";


  my $cpu_num = 1;
  while ($cpu_num le $num_of_cpus) {
    local ($user_colour, $system_colour, $idle_colour, $nice_colour)
      = split(',', $my_config->{'cpu' . $cpu_num});

# just checking to see if cpus where added while back was turned
# If there where, add databases for them
    if (!-r $minfo->{'dbtables'} . "cpu" . $cpu_num . ".rrd") {
      local ($cmd);

      $cmd = $config{'rrdtool'};
      $cmd .= " create " . $minfo->{'dbtables'} . "cpu" . $cpu_num . ".rrd";
      $cmd .= " --start 0 --step 60";
      $cmd .= " DS:user:GAUGE:100:0:U";
      $cmd .= " DS:system:GAUGE:100:0:U";
      $cmd .= " DS:nice:GAUGE:100:0:U";
      $cmd .= " DS:idle:GAUGE:100:0:U";
      $cmd .= " RRA:AVERAGE:0.5:1:70";
      $cmd .= " RRA:AVERAGE:0.5:2:750";
      $cmd .= " RRA:AVERAGE:0.5:15:650";
      $cmd .= " RRA:AVERAGE:0.5:50:650";

      `$cmd`;

      $my_config->{'cpu' . $cpu_num} = 1;
# todo: error checking
    }

    print       "<tr><td><b>CPU$cpu_num</b></td></tr>";

    print       "<tr><td><table width='100%'>";
    print        "<tr>";
    print         "<td width='15%'>";
    print          "<b>" . $text{'module_word_user'} . "</b>";
    print         "</td>";

    print         "<td width='35%'>";
    &option_colourselect('cpu' . $cpu_num . '_user', $user_colour,
			 $defaults->{'user_colour'}
			);
    print         "</td>";
    print         "<td width='15%'>";
    print          "<b>" . $text{'module_word_system'} . "</b>";
    print         "</td>";

    print         "<td width='35%'>";
    &option_colourselect('cpu' . $cpu_num . '_system', $system_colour,
			 $defaults->{'system_colour'}
			);
    print         "</td>";
    print        "</tr>";

    print        "<tr>";
    print         "<td>";
    print          "<b>" . $text{'module_word_idle'} . "</b>";
    print         "</td>";
    print         "<td>";
    &option_colourselect('cpu' . $cpu_num . '_idle', $idle_colour,
			 $defaults->{'idle_colour'}
			);
    print         "</td>";
    print         "<td>";
    print          "<b>" . $text{'module_word_nice'} . "</b>";
    print         "</td>";
    print         "<td>";
    &option_colourselect('cpu' . $cpu_num . '_nice', $nice_colour,
			 $defaults->{'nice_colour'}
			);
    print         "</td>";
    print        "</tr>";
    print       "</table></td><tr>";

    $cpu_num++;
  }

  print      "</table>";
  print  "<input type='submit' value='" . $text{'word_save'} . "'>";
  print "</form>\n";
}

sub mod_config_save {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  $my_config->{'graph_type'} = $in{'graph_type'};

  my $num_of_cpus = &find_num_cpus();

  my $cpu_num = 1;
  while ($cpu_num le $num_of_cpus) {

    $my_config->{'cpu' . $cpu_num}
      = join ( ',',
	       ($in{'cpu' . $cpu_num . '_user2_def'}
	       ? $in{'cpu' . $cpu_num . '_user2_def'}
		: $in{'cpu' . $cpu_num . '_user1_def'} eq 'default'
	       ? $defaults->{'user_colour'}
	        : $in{'cpu' . $cpu_num . '_user1_def'}),
	       ($in{'cpu' . $cpu_num . '_system2_def'}
	       ? $in{'cpu' . $cpu_num . '_system2_def'}
		: $in{'cpu' . $cpu_num . '_system1_def'} eq 'default'
	       ? $defaults->{'system_colour'}
	        : $in{'cpu' . $cpu_num . '_system1_def'}),
	       ($in{'cpu' . $cpu_num . '_idle2_def'}
	       ? $in{'cpu' . $cpu_num . '_idle2_def'}
		: $in{'cpu' . $cpu_num . '_idle1_def'} eq 'default'
	       ? $defaults->{'idle_colour'}
	        : $in{'cpu' . $cpu_num . '_idle1_def'}),
	       ($in{'cpu' . $cpu_num . '_nice2_def'}
	       ? $in{'cpu' . $cpu_num . '_nice2_def'}
		: $in{'cpu' . $cpu_num . '_nice1_def'} eq 'default'
	       ? $defaults->{'nice_colour'}
	        : $in{'cpu' . $cpu_num . '_nice1_def'})
	     );

    $cpu_num++;
  }

  &write_config($minfo, $my_config, $defaults);
}

sub write_config {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  local (%new_config, %sh_config);

  $new_config{'sample_rate'} = $my_config->{'sample_rate'};
  $sh_config{'sample_rate'} = $my_config->{'sample_rate'};
  $new_config{'runstop'} = $my_config->{'runstop'};
  $new_config{'graph_type'} = $my_config->{'graph_type'};

  my $i = 1;
  while ($my_config->{'cpu' . $i} ne '') {
    $new_config{'cpu' . $i} = $my_config->{'cpu' . $i};
    $i++;
  }

  my $n = 1;
  while ($new_config{'cpu' . $n} ne '') {
    if (!$new_config{'runstop'}) {
      $sh_config{'cpu' . $n++} = 1;
    }
  }

  unlink ($minfo->{'config_file'});
  unlink ($minfo->{'config_dir'} . 'cpu.cfg');

  &write_file($minfo->{'config_file'}, \%new_config);
  &write_file($minfo->{'config_dir'} . 'cpu.cfg', \%sh_config);
}

###############################################################################
# Start of local module functions

###############################################################################
# Find the numberof CPUs installed in this machine

sub find_num_cpus {
  my @cpu_list = split ( /\n/, `grep processor /proc/cpuinfo`);
  return scalar(@cpu_list);
}

return 1;
