#! /bin/bash

###############################################################################
#    Webmin Sysstats - cpu.sh
#
#    Webmin Sysstats Module 
#    Copyright (C) 2001 by David Bouius
#    Bug reports to: dbouius@users.sourceforge.net or gerbier@users.sourceforge.net
#    $Id: cpu.sh,v 1.14 2001/12/07 09:25:28 gerbier Exp $
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
###############################################################################

. ../../config
. cpu.cfg

trunc () {
	data=$1
	if [ "$data" -gt 100 ]
	then
		echo "100"
	else
		echo $data
	fi
}

pourcent() {
	let nb="100 * ($2 -$1)"
	let nb="$nb / $3"
	trunc $nb
}

call_cpu_info () {
	grep "cpu" /proc/stat
}

#set -x 

etat1=cpu.call1
etat2=cpu.call2

if [ ! -s $etat1 ]
then
	# first call
	call_cpu_info > $etat1
	sleep 1
fi

call_cpu_info > $etat2

let d=1

while [ $d != 0 ]
do
  let n=$d-1
  rcpu="cpu$n"
  dcpu="cpu$d"

  # bash 2 syntaxe
  #if [ -z "${!dcpu}" ]

  # bash 1 syntaxe
  eval test_cpu=\$$dcpu
  if [ -z "${test_cpu}" ]
  then
    # translate for next call
    cp -f $etat2 $etat1
    exit
  else
    sp1=$(grep ${rcpu} ${etat1})

    if [ -z "$sp1" ]
    then
	# no cpu0 in /proc/stat : seems to be 2.2 kernel, we change the rcpu filter
    	rcpu="cpu"
	sp1=$(grep ${rcpu} ${etat1})
    fi

    sp2=$(grep ${rcpu}  $etat2)


    sp1_user=$(echo $sp1 | awk '{print $2}')
    sp2_user=$(echo $sp2 | awk '{print $2}')
    sp1_syst=$(echo $sp1 | awk '{print $4}')
    sp2_syst=$(echo $sp2 | awk '{print $4}')
    sp1_nice=$(echo $sp1 | awk '{print $3}')
    sp2_nice=$(echo $sp2 | awk '{print $3}')
    sp1_idle=$(echo $sp1 | awk '{print $5}')
    sp2_idle=$(echo $sp2 | awk '{print $5}')

    let sp1_total="$sp1_user + $sp1_syst + $sp1_nice + $sp1_idle"
    let sp2_total="$sp2_user + $sp2_syst + $sp2_nice + $sp2_idle"

    # total time between 2 calls
    let tps="$sp2_total - $sp1_total"

    if [ $sp2_user -gt $sp1_user ]
    then
      let user=$(pourcent $sp1_user $sp2_user $tps)
    else
      let user=0
    fi

    if [ $sp2_syst -gt $sp1_syst ]
    then
      let syst=$(pourcent $sp1_syst  $sp2_syst $tps)
    else
      let syst=0
    fi

    if [ $sp2_nice -gt $sp1_nice ]
    then
      let nicee=$(pourcent $sp1_nice $sp2_nice $tps)
    else
      let nicee=0
    fi

    if [ $sp2_idle -gt $sp1_idle ]
    then
      let idle=$(pourcent $sp1_idle $sp2_idle  $tps)
    else
      let idle=0
    fi

    $rrdtool update ${rrdtables}${dcpu}.rrd N:$user:$syst:$nicee:$idle
  fi

  let d=d+1
done


